#include <p18cxxx.h>

#pragma config FOSC=HS, WDT=OFF, BOR = ON_ACTIVE, BORV = 1, LVP=OFF, PBADEN=OFF, XINST = OFF, MCLRE = ON

#define InstructionFrequency 16000000 / 4

#define T0Peroid 10
#define TMR0Value ( 65536 - ( InstructionFrequency / 256 / T0Peroid ))
#define TMR0HighValue ( ( TMR0Value >> 8 ) & 0xff )
#define TMR0LowValue ( TMR0Value & 0xff )

volatile unsigned char Timer0Flag = 0;

void IVTHigh(void);
void ISRHigh(void);

void main(void)
{
    LATCbits.LATC2 = 0;
    TRISCbits.TRISC2 = 0;
    LATD = 0;
    TRISD = 0;

    T0CONbits.T08BIT = 0; // 16Bits Mode
    T0CONbits.T0CS = 0; // Internal Clock
    T0CONbits.T0SE = 0;
    T0CONbits.PSA = 0; // Enable Prescale
    T0CONbits.T0PS = 0x07; // 1:256
    TMR0H = TMR0HighValue;
    TMR0L = TMR0LowValue;
    T0CONbits.TMR0ON = 1; // Timer0 Turn On

    INTCONbits.TMR0IF = 0;
    INTCONbits.TMR0IE = 1;

    ADCON0bits.CHS = 0x????; // Set to AN0
    ADCON1bits.PCFG = 0x????; // Enable Analog Mode for AN0
    ADCON0bits.ADON = 1; // ADC Module Turn On

    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1;

    while (1)
    {
        if (Timer0Flag)
        {
            Timer0Flag = 0;
            LATCbits.LATC2 = !LATCbits.LATC2;

            ADCON0bits.GO_DONE = ????;
            while (ADCON0bits.GO_DONE);
            PORTD = ADRESH;
        }
    }
}

#pragma code HighIVT = 0x0008

void IVTHigh(void)
{
    // 重新導向到ISRHigh函式(高優先權)。
    _asm goto ISRHigh _endasm
}
#pragma code

#pragma interrupt ISRHigh 

void ISRHigh(void)
{
    //在高優先權的中斷服務函式中, 添加Timer0的事件判斷。
    if (INTCONbits.TMR0IF && INTCONbits.TMR0IE)
    {
        // 清除Timer0中斷旗標。
        INTCONbits.TMR0IF = 0;
        TMR0H = TMR0HighValue;
        TMR0L = TMR0LowValue;

        Timer0Flag = 1;
    }
}
#pragma code
